@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022, Alliance for Open Media. All rights reserved
+ * Copyright (c) 2022, Alliance for Open Media. All rights reserved.
  *
  * This source code is subject to the terms of the BSD 2 Clause License and
  * the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -16,12 +16,12 @@
 #include "aom/aom_integer.h"
 #include "aom_dsp/arm/transpose_neon.h"
 
-static INLINE int16x4_t clip3_s16(const int16x4_t val, const int16x4_t low,
+static inline int16x4_t clip3_s16(const int16x4_t val, const int16x4_t low,
                                   const int16x4_t high) {
   return vmin_s16(vmax_s16(val, low), high);
 }
 
-static INLINE uint16x8_t convert_to_unsigned_pixel_u16(int16x8_t val,
+static inline uint16x8_t convert_to_unsigned_pixel_u16(int16x8_t val,
                                                        int bitdepth) {
   const int16x8_t low = vdupq_n_s16(0);
   const uint16x8_t high = vdupq_n_u16((1 << bitdepth) - 1);
@@ -30,14 +30,14 @@ static INLINE uint16x8_t convert_to_unsigned_pixel_u16(int16x8_t val,
 }
 
 // (abs(p1 - p0) > thresh) || (abs(q1 - q0) > thresh)
-static INLINE uint16x4_t hev(const uint16x8_t abd_p0p1_q0q1,
+static inline uint16x4_t hev(const uint16x8_t abd_p0p1_q0q1,
                              const uint16_t thresh) {
   const uint16x8_t a = vcgtq_u16(abd_p0p1_q0q1, vdupq_n_u16(thresh));
   return vorr_u16(vget_low_u16(a), vget_high_u16(a));
 }
 
 // abs(p0 - q0) * 2 + abs(p1 - q1) / 2 <= outer_thresh
-static INLINE uint16x4_t outer_threshold(const uint16x4_t p1,
+static inline uint16x4_t outer_threshold(const uint16x4_t p1,
                                          const uint16x4_t p0,
                                          const uint16x4_t q0,
                                          const uint16x4_t q1,
@@ -52,7 +52,7 @@ static INLINE uint16x4_t outer_threshold(const uint16x4_t p1,
 
 // abs(p1 - p0) <= inner_thresh && abs(q1 - q0) <= inner_thresh &&
 //   outer_threshold()
-static INLINE uint16x4_t needs_filter4(const uint16x8_t abd_p0p1_q0q1,
+static inline uint16x4_t needs_filter4(const uint16x8_t abd_p0p1_q0q1,
                                        const uint16_t inner_thresh,
                                        const uint16x4_t outer_mask) {
   const uint16x8_t a = vcleq_u16(abd_p0p1_q0q1, vdupq_n_u16(inner_thresh));
@@ -63,7 +63,7 @@ static INLINE uint16x4_t needs_filter4(const uint16x8_t abd_p0p1_q0q1,
 // abs(p2 - p1) <= inner_thresh && abs(p1 - p0) <= inner_thresh &&
 //   abs(q1 - q0) <= inner_thresh && abs(q2 - q1) <= inner_thresh &&
 //   outer_threshold()
-static INLINE uint16x4_t needs_filter6(const uint16x8_t abd_p0p1_q0q1,
+static inline uint16x4_t needs_filter6(const uint16x8_t abd_p0p1_q0q1,
                                        const uint16x8_t abd_p1p2_q1q2,
                                        const uint16_t inner_thresh,
                                        const uint16x4_t outer_mask) {
@@ -77,7 +77,7 @@ static INLINE uint16x4_t needs_filter6(const uint16x8_t abd_p0p1_q0q1,
 //   abs(p1 - p0) <= inner_thresh && abs(q1 - q0) <= inner_thresh &&
 //   abs(q2 - q1) <= inner_thresh && abs(q3 - q2) <= inner_thresh
 //   outer_threshold()
-static INLINE uint16x4_t needs_filter8(const uint16x8_t abd_p0p1_q0q1,
+static inline uint16x4_t needs_filter8(const uint16x8_t abd_p0p1_q0q1,
                                        const uint16x8_t abd_p1p2_q1q2,
                                        const uint16x8_t abd_p2p3_q2q3,
                                        const uint16_t inner_thresh,
@@ -92,7 +92,7 @@ static INLINE uint16x4_t needs_filter8(const uint16x8_t abd_p0p1_q0q1,
 // -----------------------------------------------------------------------------
 // filterN_masks functions.
 
-static INLINE void filter4_masks(const uint16x8_t p0q0, const uint16x8_t p1q1,
+static inline void filter4_masks(const uint16x8_t p0q0, const uint16x8_t p1q1,
                                  const uint16_t hev_thresh,
                                  const uint16x4_t outer_mask,
                                  const uint16_t inner_thresh,
@@ -112,7 +112,7 @@ static INLINE void filter4_masks(const uint16x8_t p0q0, const uint16x8_t p1q1,
 // abs(p1 - p0) <= flat_thresh && abs(q1 - q0) <= flat_thresh &&
 //   abs(p2 - p0) <= flat_thresh && abs(q2 - q0) <= flat_thresh
 // |flat_thresh| == 4 for 10 bit decode.
-static INLINE uint16x4_t is_flat3(const uint16x8_t abd_p0p1_q0q1,
+static inline uint16x4_t is_flat3(const uint16x8_t abd_p0p1_q0q1,
                                   const uint16x8_t abd_p0p2_q0q2,
                                   const int bitdepth) {
   const int flat_thresh = 1 << (bitdepth - 8);
@@ -121,7 +121,7 @@ static INLINE uint16x4_t is_flat3(const uint16x8_t abd_p0p1_q0q1,
   return vand_u16(vget_low_u16(b), vget_high_u16(b));
 }
 
-static INLINE void filter6_masks(
+static inline void filter6_masks(
     const uint16x8_t p2q2, const uint16x8_t p1q1, const uint16x8_t p0q0,
     const uint16_t hev_thresh, const uint16x4_t outer_mask,
     const uint16_t inner_thresh, const int bitdepth,
@@ -139,7 +139,7 @@ static INLINE void filter6_masks(
 //   abs(p[N+1] - p0) <= flat_thresh && abs(q[N+1] - q0) <= flat_thresh &&
 //   abs(p[N+2] - p0) <= flat_thresh && abs(q[N+1] - q0) <= flat_thresh
 // |flat_thresh| == 4 for 10 bit decode.
-static INLINE uint16x4_t is_flat4(const uint16x8_t abd_pnp0_qnq0,
+static inline uint16x4_t is_flat4(const uint16x8_t abd_pnp0_qnq0,
                                   const uint16x8_t abd_pn1p0_qn1q0,
                                   const uint16x8_t abd_pn2p0_qn2q0,
                                   const int bitdepth) {
@@ -150,7 +150,7 @@ static INLINE uint16x4_t is_flat4(const uint16x8_t abd_pnp0_qnq0,
   return vand_u16(vget_low_u16(c), vget_high_u16(c));
 }
 
-static INLINE void filter8_masks(
+static inline void filter8_masks(
     const uint16x8_t p3q3, const uint16x8_t p2q2, const uint16x8_t p1q1,
     const uint16x8_t p0q0, const uint16_t hev_thresh,
     const uint16x4_t outer_mask, const uint16_t inner_thresh,
@@ -175,7 +175,7 @@ static INLINE void filter8_masks(
 // filterN functions.
 
 // Calculate filter4() or filter2() based on |hev_mask|.
-static INLINE void filter4(const uint16x8_t p0q0, const uint16x8_t p0q1,
+static inline void filter4(const uint16x8_t p0q0, const uint16x8_t p0q1,
                            const uint16x8_t p1q1, const uint16x4_t hev_mask,
                            int bitdepth, uint16x8_t *const p1q1_result,
                            uint16x8_t *const p0q0_result) {
@@ -298,7 +298,7 @@ void aom_highbd_lpf_vertical_4_neon(uint16_t *s, int pitch,
 
   uint16x4_t src[4] = { vld1_u16(dst_p1), vld1_u16(dst_p0), vld1_u16(dst_q0),
                         vld1_u16(dst_q1) };
-  transpose_u16_4x4(src);
+  transpose_array_inplace_u16_4x4(src);
 
   // Adjust thresholds to bitdepth.
   const int outer_thresh = *blimit << (bd - 8);
@@ -344,7 +344,7 @@ void aom_highbd_lpf_vertical_4_neon(uint16_t *s, int pitch,
     vget_high_u16(p0q0_output),
     vget_high_u16(p1q1_output),
   };
-  transpose_u16_4x4(output);
+  transpose_array_inplace_u16_4x4(output);
 
   vst1_u16(dst_p1, output[0]);
   vst1_u16(dst_p0, output[1]);
@@ -361,7 +361,7 @@ void aom_highbd_lpf_vertical_4_dual_neon(
                                  bd);
 }
 
-static INLINE void filter6(const uint16x8_t p2q2, const uint16x8_t p1q1,
+static inline void filter6(const uint16x8_t p2q2, const uint16x8_t p1q1,
                            const uint16x8_t p0q0, uint16x8_t *const p1q1_output,
                            uint16x8_t *const p0q0_output) {
   // Sum p1 and q1 output from opposite directions.
@@ -386,7 +386,7 @@ static INLINE void filter6(const uint16x8_t p2q2, const uint16x8_t p1q1,
   // p1q1 = p2q2 + 2 * (p2q2 + p1q1 + p0q0) + q0p0
   //        ^^^^^^                          ^^^^^^
   // Should dual issue with the left shift.
-  const uint16x8_t q0p0 = transpose64_u16q(p0q0);
+  const uint16x8_t q0p0 = vextq_u16(p0q0, p0q0, 4);
   const uint16x8_t outer_sum = vaddq_u16(p2q2, q0p0);
   sum = vaddq_u16(sum, outer_sum);
 
@@ -401,7 +401,7 @@ static INLINE void filter6(const uint16x8_t p2q2, const uint16x8_t p1q1,
   // p0q0 = p1q1 - (2 * p2q2) + q0p0 + q1p1
   //        ^^^^^^^^
   sum = vsubq_u16(sum, p2q2_double);
-  const uint16x8_t q1p1 = transpose64_u16q(p1q1);
+  const uint16x8_t q1p1 = vextq_u16(p1q1, p1q1, 4);
   sum = vaddq_u16(sum, vaddq_u16(q0p0, q1p1));
 
   *p0q0_output = vrshrq_n_u16(sum, 3);
@@ -505,7 +505,7 @@ void aom_highbd_lpf_vertical_6_neon(uint16_t *s, int pitch,
   // and src_raw[3] after transpose.
   uint16x8_t src_raw[4] = { vld1q_u16(dst_0), vld1q_u16(dst_1),
                             vld1q_u16(dst_2), vld1q_u16(dst_3) };
-  transpose_u16_4x8q(src_raw);
+  transpose_array_inplace_u16_4x8(src_raw);
   // p2, p1, p0, q0, q1, q2
   const uint16x4_t src[6] = {
     vget_low_u16(src_raw[0]),  vget_low_u16(src_raw[1]),
@@ -574,7 +574,7 @@ void aom_highbd_lpf_vertical_6_neon(uint16_t *s, int pitch,
     vget_high_u16(p0q0_output),
     vget_high_u16(p1q1_output),
   };
-  transpose_u16_4x4(output);
+  transpose_array_inplace_u16_4x4(output);
 
   // dst_n starts at p2, so adjust to p1.
   vst1_u16(dst_0 + 1, output[0]);
@@ -592,7 +592,7 @@ void aom_highbd_lpf_vertical_6_dual_neon(
                                  bd);
 }
 
-static INLINE void filter8(const uint16x8_t p3q3, const uint16x8_t p2q2,
+static inline void filter8(const uint16x8_t p3q3, const uint16x8_t p2q2,
                            const uint16x8_t p1q1, const uint16x8_t p0q0,
                            uint16x8_t *const p2q2_output,
                            uint16x8_t *const p1q1_output,
@@ -626,7 +626,7 @@ static INLINE void filter8(const uint16x8_t p3q3, const uint16x8_t p2q2,
 
   // p2q2 = p3q3 + 2 * (p3q3 + p2q2) + p1q1 + p0q0 + q0p0
   //                                               ^^^^^^
-  const uint16x8_t q0p0 = transpose64_u16q(p0q0);
+  const uint16x8_t q0p0 = vextq_u16(p0q0, p0q0, 4);
   sum = vaddq_u16(sum, q0p0);
 
   *p2q2_output = vrshrq_n_u16(sum, 3);
@@ -635,7 +635,7 @@ static INLINE void filter8(const uint16x8_t p3q3, const uint16x8_t p2q2,
   // p1 = p2 - p3 - p2 + p1 + q1
   // q1 = q2 - q3 - q2 + q0 + p1
   sum = vsubq_u16(sum, p23q23);
-  const uint16x8_t q1p1 = transpose64_u16q(p1q1);
+  const uint16x8_t q1p1 = vextq_u16(p1q1, p1q1, 4);
   sum = vaddq_u16(sum, vaddq_u16(p1q1, q1p1));
 
   *p1q1_output = vrshrq_n_u16(sum, 3);
@@ -644,7 +644,7 @@ static INLINE void filter8(const uint16x8_t p3q3, const uint16x8_t p2q2,
   // p0 = p1 - p3 - p1 + p0 + q2
   // q0 = q1 - q3 - q1 + q0 + p2
   sum = vsubq_u16(sum, vaddq_u16(p3q3, p1q1));
-  const uint16x8_t q2p2 = transpose64_u16q(p2q2);
+  const uint16x8_t q2p2 = vextq_u16(p2q2, p2q2, 4);
   sum = vaddq_u16(sum, vaddq_u16(p0q0, q2p2));
 
   *p0q0_output = vrshrq_n_u16(sum, 3);
@@ -742,7 +742,7 @@ void aom_highbd_lpf_horizontal_8_dual_neon(
   aom_highbd_lpf_horizontal_8_neon(s + 4, pitch, blimit1, limit1, thresh1, bd);
 }
 
-static INLINE uint16x8_t reverse_low_half(const uint16x8_t a) {
+static inline uint16x8_t reverse_low_half(const uint16x8_t a) {
   return vcombine_u16(vrev64_u16(vget_low_u16(a)), vget_high_u16(a));
 }
 
@@ -827,7 +827,7 @@ void aom_highbd_lpf_vertical_8_neon(uint16_t *s, int pitch,
   uint16x8_t output[4] = { p0q0_output, p1q1_output, p2q2_output, p3q3 };
   // After transpose, |output| will contain rows of the form:
   // p0 p1 p2 p3 q0 q1 q2 q3
-  transpose_u16_4x8q(output);
+  transpose_array_inplace_u16_4x8(output);
 
   // Reverse p values to produce original order:
   // p3 p2 p1 p0 q0 q1 q2 q3
@@ -846,7 +846,7 @@ void aom_highbd_lpf_vertical_8_dual_neon(
                                  bd);
 }
 
-static INLINE void filter14(
+static inline void filter14(
     const uint16x8_t p6q6, const uint16x8_t p5q5, const uint16x8_t p4q4,
     const uint16x8_t p3q3, const uint16x8_t p2q2, const uint16x8_t p1q1,
     const uint16x8_t p0q0, uint16x8_t *const p5q5_output,
@@ -883,7 +883,7 @@ static INLINE void filter14(
   //                                                           ^^
   // q5 = p0 + q0 + q1 + q2 + q3 + (2 * q4) + (2 * q5) + (7 * q6)
   //      ^^
-  const uint16x8_t q0p0 = transpose64_u16q(p0q0);
+  const uint16x8_t q0p0 = vextq_u16(p0q0, p0q0, 4);
   sum = vaddq_u16(sum, q0p0);
 
   *p5q5_output = vrshrq_n_u16(sum, 4);
@@ -892,7 +892,7 @@ static INLINE void filter14(
   // p4 = p5 - (2 * p6) + p3 + q1
   // q4 = q5 - (2 * q6) + q3 + p1
   sum = vsubq_u16(sum, vshlq_n_u16(p6q6, 1));
-  const uint16x8_t q1p1 = transpose64_u16q(p1q1);
+  const uint16x8_t q1p1 = vextq_u16(p1q1, p1q1, 4);
   sum = vaddq_u16(vaddq_u16(p3q3, q1p1), sum);
 
   *p4q4_output = vrshrq_n_u16(sum, 4);
@@ -901,7 +901,7 @@ static INLINE void filter14(
   // p3 = p4 - p6 - p5 + p2 + q2
   // q3 = q4 - q6 - q5 + q2 + p2
   sum = vsubq_u16(sum, vaddq_u16(p6q6, p5q5));
-  const uint16x8_t q2p2 = transpose64_u16q(p2q2);
+  const uint16x8_t q2p2 = vextq_u16(p2q2, p2q2, 4);
   sum = vaddq_u16(vaddq_u16(p2q2, q2p2), sum);
 
   *p3q3_output = vrshrq_n_u16(sum, 4);
@@ -910,7 +910,7 @@ static INLINE void filter14(
   // p2 = p3 - p6 - p4 + p1 + q3
   // q2 = q3 - q6 - q4 + q1 + p3
   sum = vsubq_u16(sum, vaddq_u16(p6q6, p4q4));
-  const uint16x8_t q3p3 = transpose64_u16q(p3q3);
+  const uint16x8_t q3p3 = vextq_u16(p3q3, p3q3, 4);
   sum = vaddq_u16(vaddq_u16(p1q1, q3p3), sum);
 
   *p2q2_output = vrshrq_n_u16(sum, 4);
@@ -919,7 +919,7 @@ static INLINE void filter14(
   // p1 = p2 - p6 - p3 + p0 + q4
   // q1 = q2 - q6 - q3 + q0 + p4
   sum = vsubq_u16(sum, vaddq_u16(p6q6, p3q3));
-  const uint16x8_t q4p4 = transpose64_u16q(p4q4);
+  const uint16x8_t q4p4 = vextq_u16(p4q4, p4q4, 4);
   sum = vaddq_u16(vaddq_u16(p0q0, q4p4), sum);
 
   *p1q1_output = vrshrq_n_u16(sum, 4);
@@ -928,7 +928,7 @@ static INLINE void filter14(
   // p0 = p1 - p6 - p2 + q0 + q5
   // q0 = q1 - q6 - q2 + p0 + p5
   sum = vsubq_u16(sum, vaddq_u16(p6q6, p2q2));
-  const uint16x8_t q5p5 = transpose64_u16q(p5q5);
+  const uint16x8_t q5p5 = vextq_u16(p5q5, p5q5, 4);
   sum = vaddq_u16(vaddq_u16(q0p0, q5p5), sum);
 
   *p0q0_output = vrshrq_n_u16(sum, 4);
@@ -1080,7 +1080,7 @@ void aom_highbd_lpf_horizontal_14_dual_neon(
   aom_highbd_lpf_horizontal_14_neon(s + 4, pitch, blimit1, limit1, thresh1, bd);
 }
 
-static INLINE uint16x8x2_t permute_acdb64(const uint16x8_t ab,
+static inline uint16x8x2_t permute_acdb64(const uint16x8_t ab,
                                           const uint16x8_t cd) {
   uint16x8x2_t acdb;
 #if AOM_ARCH_AARCH64
@@ -1118,14 +1118,14 @@ void aom_highbd_lpf_vertical_14_neon(uint16_t *s, int pitch,
   uint16x8_t src_p[4] = { vld1q_u16(dst_0), vld1q_u16(dst_1), vld1q_u16(dst_2),
                           vld1q_u16(dst_3) };
   // p7 will be the low half of src_p[0]. Not used until the end.
-  transpose_u16_4x8q(src_p);
+  transpose_array_inplace_u16_4x8(src_p);
 
   // Low halves:  q0 q1 q2 q3
   // High halves: q4 q5 q6 q7
   uint16x8_t src_q[4] = { vld1q_u16(dst_0 + 8), vld1q_u16(dst_1 + 8),
                           vld1q_u16(dst_2 + 8), vld1q_u16(dst_3 + 8) };
   // q7 will be the high half of src_q[3]. Not used until the end.
-  transpose_u16_4x8q(src_q);
+  transpose_array_inplace_u16_4x8(src_q);
 
   // Adjust thresholds to bitdepth.
   const int outer_thresh = *blimit << (bd - 8);
@@ -1238,10 +1238,10 @@ void aom_highbd_lpf_vertical_14_neon(uint16_t *s, int pitch,
   const uint16x8x2_t p4p0_q0q4 = permute_acdb64(p4q4_output, p0q0_output);
   uint16x8_t output_p[4] = { p7p3_q3q7.val[0], p6p2_q2q6.val[0],
                              p5p1_q1q5.val[0], p4p0_q0q4.val[0] };
-  transpose_u16_4x8q(output_p);
+  transpose_array_inplace_u16_4x8(output_p);
   uint16x8_t output_q[4] = { p4p0_q0q4.val[1], p5p1_q1q5.val[1],
                              p6p2_q2q6.val[1], p7p3_q3q7.val[1] };
-  transpose_u16_4x8q(output_q);
+  transpose_array_inplace_u16_4x8(output_q);
 
   // Reverse p values to produce original order:
   // p3 p2 p1 p0 q0 q1 q2 q3
