@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, Alliance for Open Media. All rights reserved
+ * Copyright (c) 2016, Alliance for Open Media. All rights reserved.
  *
  * This source code is subject to the terms of the BSD 2 Clause License and
  * the Alliance for Open Media Patent License 1.0. If the BSD 2 Clause License
@@ -190,9 +190,15 @@ typedef struct {
   int sframe_due;
 
   int high_source_sad;
+  int high_motion_content_screen_rtc;
   uint64_t avg_source_sad;
   uint64_t prev_avg_source_sad;
   uint64_t frame_source_sad;
+  uint64_t spatial_variance_keyframe;
+  int last_encoded_size_keyframe;
+  int last_target_size_keyframe;
+  int frames_since_scene_change;
+  int perc_flat_blocks_keyframe;
 
   int avg_frame_bandwidth;  // Average frame size target for clip
   int min_frame_bandwidth;  // Minimum allocation used for any frame
@@ -205,6 +211,10 @@ typedef struct {
   int decimation_factor;
   int decimation_count;
   int prev_frame_is_dropped;
+  int drop_count_consec;
+  int max_consec_drop;
+  int force_max_q;
+  int postencode_drop;
 
   /*!
    * Frame number for encoded frames (non-dropped).
@@ -247,6 +257,9 @@ typedef struct {
   // signals if number of blocks with motion is high
   int percent_blocks_with_motion;
 
+  // signals percentage of 16x16 blocks that are inactive, via active_maps
+  int percent_blocks_inactive;
+
   // Maximum value of source sad across all blocks of frame.
   uint64_t max_block_source_sad;
 
@@ -673,10 +686,6 @@ int av1_rc_regulate_q(const struct AV1_COMP *cpi, int target_bits_per_frame,
                       int width, int height);
 
 /*!\cond */
-// Gets the appropriate bpmb ennumerator based on the frame and content type
-int av1_get_bpmb_enumerator(FRAME_TYPE frame_type,
-                            const int is_screen_content_type);
-
 // Estimates bits per mb for a given qindex and correction factor.
 int av1_rc_bits_per_mb(const struct AV1_COMP *cpi, FRAME_TYPE frame_type,
                        int qindex, double correction_factor,
@@ -686,7 +695,7 @@ int av1_rc_bits_per_mb(const struct AV1_COMP *cpi, FRAME_TYPE frame_type,
 int av1_rc_clamp_iframe_target_size(const struct AV1_COMP *const cpi,
                                     int64_t target);
 int av1_rc_clamp_pframe_target_size(const struct AV1_COMP *const cpi,
-                                    int target, uint8_t frame_update_type);
+                                    int64_t target, uint8_t frame_update_type);
 
 // Find q_index corresponding to desired_q, within [best_qindex, worst_qindex].
 // To be precise, 'q_index' is the smallest integer, for which the corresponding
@@ -706,8 +715,6 @@ int av1_compute_qdelta_by_rate(const struct AV1_COMP *cpi,
                                FRAME_TYPE frame_type, int qindex,
                                double rate_target_ratio);
 
-int av1_frame_type_qdelta(const struct AV1_COMP *cpi, int q);
-
 void av1_rc_update_framerate(struct AV1_COMP *cpi, int width, int height);
 
 void av1_rc_set_gf_interval_range(const struct AV1_COMP *const cpi,
@@ -817,6 +824,19 @@ void av1_get_one_pass_rt_params(struct AV1_COMP *cpi,
  */
 int av1_encodedframe_overshoot_cbr(struct AV1_COMP *cpi, int *q);
 
+/*!\brief Check if frame should be dropped, for RTC mode.
+ *
+ * \ingroup rate_control
+ * \param[in]       cpi          Top level encoder structure
+ * \param[in,out]       size         Size of encoded frame
+ *
+ * \return 1 if frame is to be dropped, 0 otherwise (no drop).
+ * Set cpi->rc.force_max_q if frame is to be dropped, and updates are
+ * made to rate control parameters. *size is set to 0 when this
+ * function returns 1 (frame is dropped).
+ */
+int av1_postencode_drop_cbr(struct AV1_COMP *cpi, size_t *size);
+
 /*!\brief Compute the q_indices for a single frame.
  *
  * Intended to be used with AOM_Q mode.
@@ -831,30 +851,6 @@ int av1_encodedframe_overshoot_cbr(struct AV1_COMP *cpi, int *q);
 int av1_q_mode_get_q_index(int base_q_index, int gf_update_type,
                            int gf_pyramid_level, int arf_q);
 
-/*!\brief Compute the q_indices for the ARF of a GOP.
- *
- * \param[in]       base_q_index      Base q index
- * \param[in]       gfu_boost         GFU boost
- * \param[in]       bit_depth         Bit depth
- * \param[in]       arf_boost_factor  ARF boost factor
- *
- * \return Returns the q_index for the ARF frame.
- */
-int av1_get_arf_q_index(int base_q_index, int gfu_boost, int bit_depth,
-                        double arf_boost_factor);
-
-#if !CONFIG_REALTIME_ONLY
-struct TplDepFrame;
-/*!\brief Compute the q_indices for the ARF of a GOP in Q mode.
- *
- * \param[in]       cpi               Top level encoder structure
- * \param[in]       tpl_frame         Tpl Frame stats
- *
- * \return Returns the q_index for the ARF frame.
- */
-int av1_get_arf_q_index_q_mode(struct AV1_COMP *cpi,
-                               struct TplDepFrame *tpl_frame);
-#endif
 #ifdef __cplusplus
 }  // extern "C"
 #endif
